// Traveling Salesman Problem via Genetic Algorithm
// Written by David J. Stein, Esq., Fall 2004. See "License.html" for terms and conditions of use.
//
// This module contains the general-purpose Genetic Algorithm class.

#region Using declarations

using System;
using System.Collections;

#endregion

namespace GeneticAlgorithmForTravelingSalesmanProblem {

	#region Delegate declarations (to be plugged in by each adapter subclass)

	public delegate Entity CreateFunctionDelegate();
	public delegate bool VerifyFunctionDelegate(Entity e);
	public delegate double FitnessFunctionDelegate(Entity e);
	public delegate void EntityFunctionDelegate(Entity e);
	public delegate void MutateFunctionDelegate(Entity e);

	#endregion

	public class GeneticAlgorithmModel {

		#region Variables and initialization code

		public ArrayList lstEntities;
		public FitnessFunctionDelegate fFitnessFunction = null;
		public CreateFunctionDelegate fCreateFunction = null;
		public VerifyFunctionDelegate fVerifyFunction = null;
		public EntityFunctionDelegate fEntityFunction = null;
		public MutateFunctionDelegate fMutateFunction = null;
		public int iPopulationSize = 100;
		public double dMutationRate = 0.05;
		public double dNewEntities = 0.05;
		public double dReplacementRate = 1.0;
		public double dBuddingRate = 0.0;
		public int iEntitySize = 1024;
		public Random random;

		public GeneticAlgorithmModel() {
			lstEntities = new ArrayList();
			random = new Random((int) System.DateTime.Now.Ticks);
			this.fCreateFunction = new CreateFunctionDelegate(CreateFunction_Default);
			this.fVerifyFunction = new VerifyFunctionDelegate(VerifyFunction_AlwaysOK);
			this.fEntityFunction = new EntityFunctionDelegate(EntityFunction_DoNothing);
			this.fFitnessFunction = new FitnessFunctionDelegate(FitnessFunction_Random);
			this.fMutateFunction = new MutateFunctionDelegate(MutateFunction_Random);
		}

		#endregion

		#region Core functions

		public void ReproducePopulation() {

			ArrayList lstChildEntities = new ArrayList();

			// compute fitnesses
			double totalFitness = 0.0;
			foreach (Entity e in lstEntities) {
				e.dFitness = fFitnessFunction(e);
				totalFitness += e.dFitness;
			}

			// keep top entities
			int iKeep = (int) ((1.0 - dReplacementRate) * iPopulationSize);
			ArrayList lstKeptParents = new ArrayList();
			if (dReplacementRate < 1.0) {
				foreach (Entity e in lstEntities) {
					if (lstKeptParents.Count < iKeep)
						lstKeptParents.Add(e);
					else if (e.dFitness > ((Entity) lstKeptParents[iKeep - 1]).dFitness) {
						for (int i = iKeep - 2; i >= 0; i--) {
							if (((Entity) lstKeptParents[i]).dFitness >= e.dFitness) {  // insert here
								lstKeptParents.Insert(i, e);
								break;
							}
						}
						if (lstKeptParents.Count == iKeep)  // entity is actually the top entity - add at top of list
							lstKeptParents.Insert(0, e);
						lstKeptParents.RemoveAt(iKeep);
					}
				}
			}

			int iChildren = (int) (iPopulationSize * (1.0 - dNewEntities)) - iKeep;
			for (int k = 0; k < iChildren; k++) {
				// budding
				if (random.NextDouble() < dBuddingRate) {
					double dParent = random.NextDouble() * totalFitness;
					Entity eParent = null;
					for (int i = 0; i < iPopulationSize; i++) {
						Entity e = (Entity) lstEntities[i];
						if (dParent <= e.dFitness) {
							eParent = e;
							break;
						}
						else
							dParent -= e.dFitness;
					}
					if (eParent != null) {
						Entity eChild = null;
						do {
							eChild = fCreateFunction();
							int iBreakStart = random.Next(iEntitySize);
							int iBreakEnd = random.Next(iEntitySize);
						while (iBreakEnd == iBreakStart)
							iBreakEnd = random.Next(iEntitySize);
							if (iBreakEnd > iBreakStart) {
								for (int j = iBreakStart; j < iBreakEnd; j++)
									eChild.contents[j] = eParent.contents[j];
							}
							else {
								for (int j = iBreakStart; j < iEntitySize; j++)
									eChild.contents[j] = eParent.contents[j];
								for (int j = 0; j < iBreakStart; j++)
									eChild.contents[j] = eParent.contents[j];
							}
						} while (fVerifyFunction(eChild) == false);

						// add to child list
						lstChildEntities.Add(eChild);
					}
				}

				else {  // true crossover
					double dFather = random.NextDouble() * totalFitness;
					double dMother = random.NextDouble() * totalFitness;
					Entity eFather = null, eMother = null;
					for (int i = 0; i < iPopulationSize; i++) {
						Entity e = (Entity) lstEntities[i];
						if (dFather >= 0.0) {
							if (dFather <= e.dFitness) {
								eFather = e;
								dFather = -1;  // stop looking
								if (dMother == -1)
									break;
							}
							else
								dFather -= e.dFitness;
						}
						if (dMother >= 0.0) {
							if (dMother <= e.dFitness) {
								eMother = e;
								dMother = -1;  // stop looking
								if (dFather == -1)
									break;
							}
							else
								dMother -= e.dFitness;
						}
					}
					if ((eMother != null) && (eFather != null)) {
						Entity eChild = null;
						do {
							eChild = fCreateFunction();
							int iBreakStart = random.Next(iEntitySize);
							int iBreakEnd = random.Next(iEntitySize);
						while (iBreakEnd == iBreakStart)
							iBreakEnd = random.Next(iEntitySize);
							if (iBreakEnd > iBreakStart) {
								for (int j = iBreakStart; j < iBreakEnd; j++)
									eChild.contents[j] = eMother.contents[j];
								for (int j = 0; j < iBreakStart; j++)
									eChild.contents[j] = eFather.contents[j];
								for (int j = iBreakEnd; j < iEntitySize; j++)
									eChild.contents[j] = eFather.contents[j];
							}
							else {
								for (int j = iBreakEnd; j < iBreakStart; j++)
									eChild.contents[j] = eFather.contents[j];
								for (int j = iBreakStart; j < iEntitySize; j++)
									eChild.contents[j] = eMother.contents[j];
								for (int j = 0; j < iBreakStart; j++)
									eChild.contents[j] = eMother.contents[j];
							}
						} while (fVerifyFunction(eChild) == false);

						// add to child list
						lstChildEntities.Add(eChild);
					}
				}
			}
			while (lstChildEntities.Count < iPopulationSize)
				lstChildEntities.Add(fCreateFunction());

			// mutations
			for (int i = 0; i < (int) (iPopulationSize * dMutationRate); i++) {
				Entity eMutate = (Entity) lstChildEntities[random.Next(iPopulationSize)];
				fMutateFunction(eMutate);
			}

			// add in kept parents
			for (int i = lstKeptParents.Count - 1; i >= 0; i--)
				lstChildEntities.Insert(0, lstKeptParents[i]);

			// set child entities list to master list and exit
			lstEntities = lstChildEntities;
		}

		#endregion

		#region Default delegate functions and delegate assignment functions

		public Entity CreateFunction_Default() {
			Entity e = new Entity(iEntitySize);
			for (int i = 0; i < iEntitySize; i++)
				e.contents[i] = (char) random.Next(256);
			return e;
		}

		public void MutateFunction_Random(Entity e) {
			e.contents[random.Next(iEntitySize)] = (char) random.Next(256);
		}

		public bool VerifyFunction_AlwaysOK(Entity e) {
			return true;
		}

		public double FitnessFunction_Random(Entity e) {
			return random.NextDouble();
		}

		public void EntityFunction_DoNothing(Entity e) {
		}

		public void SetCreateFunction(CreateFunctionDelegate fCreateFunction) {
			this.fCreateFunction = fCreateFunction;
		}

		public void SetVerifyFunction(VerifyFunctionDelegate fVerifyFunction) {
			this.fVerifyFunction = fVerifyFunction;
		}

		public void SetFitnessFunction(FitnessFunctionDelegate fFitnessFunction) {
			this.fFitnessFunction = fFitnessFunction;
		}

		public void SetEntityFunction(EntityFunctionDelegate fEntityFunction) {
			this.fEntityFunction = fEntityFunction;
		}

		public void SetMutateFunction(MutateFunctionDelegate fMutateFunction) {
			this.fMutateFunction = fMutateFunction;
		}

		#endregion

		#region Helper functions

		public void Initialize() {
			lstEntities = new ArrayList();
			while (lstEntities.Count < iPopulationSize) {
				Entity e = this.fCreateFunction();
				if (fVerifyFunction(e) == true)
					lstEntities.Add(e);
			}
		}

		#endregion

	}

	public class Entity {

		#region Variables and initialization code

		public char[] contents = null;
		public double dFitness;

		public Entity() {
			contents = new char[1024];  // create 1,024-byte array by default
		}

		public Entity(int size) {
			contents = new char[size];
		}

		#endregion

	}
}
