// Traveling Salesman Problem via Genetic Algorithm
// Written by David J. Stein, Esq., Fall 2004. See "License.html" for terms and conditions of use.
//
// This module contains the adapter for the general-purpose genetic algorithm for for the Traveling Salesman Problem.

#region Using declarations

using System;
using System.Collections;

#endregion

namespace GeneticAlgorithmForTravelingSalesmanProblem {

	public class TravelingSalesmanSolution	: GeneticAlgorithmModel {

		// This class adapts the GeneticAlgorithmModel class as a TravelingSalesmanSolution.

		#region Variables and initialization code

		public int iNumCities;
		public int[,] iDistances;
		public int iMaxTrip = 1;
		public ArrayList cities;

		public TravelingSalesmanSolution()	{
			SetCreateFunction(new CreateFunctionDelegate(CreateRoute));
			SetVerifyFunction(new VerifyFunctionDelegate(VerifyRoute));
			SetFitnessFunction(new FitnessFunctionDelegate(MeasureFitness));			
			SetMutateFunction(new MutateFunctionDelegate(MutateRoute));
			iNumCities = this.iEntitySize = 0;
			iPopulationSize = 100;
			dMutationRate = 0.10;
			dReplacementRate = 0.95;
			dBuddingRate = 0.5;
			dNewEntities = 0.0;
			cities = new ArrayList();
		}

		#endregion

		#region Delegate functions

		public Entity CreateRoute() {
			Route r = new Route(iNumCities);
			// create random route
			bool[] visited = new bool[iNumCities];
			for (int i = 0; i < iNumCities; i++)
				visited[i] = false;
			for (int i = 0; i < iNumCities; i++) {
				int city = random.Next(iNumCities);
				while (visited[city] == true)
					city = random.Next(iNumCities);
				r.contents[i] = (char) city;
				visited[city] = true;
			}
			return r;
		}

		public bool VerifyRoute(Entity e) {
			bool correct = true;
			// create checklist
			bool[] visited = new bool[iNumCities];
			for (int i = 0; i < iNumCities; i++)
				visited[i] = false;
			for (int i = 0; i < iNumCities; i++) {
				if (visited[(int) (e.contents[i])] == true)
					correct = false;
				else
					visited[(int) (e.contents[i])] = true;
			}
			return correct;
		}

		public double MeasureFitness(Entity e) {
			return iMaxTrip - ((Route) e).MeasureDistance(iDistances, iMaxTrip);
		}

		public void MutateRoute(Entity e) {
			Route r = (Route) e;
			// swap two cities
			int c1 = random.Next(cities.Count);
			int c2 = random.Next(cities.Count);
			while (c1 == c2)
				c2 = random.Next(cities.Count);
			char swap = e.contents[c1];
			e.contents[c1] = e.contents[c2];
			e.contents[c2] = swap;
		}

		#endregion

		#region Helper functions

		public void Clear() {
			iNumCities = iEntitySize = 0;
			this.cities = new ArrayList();
		}

		public void AddCity(int x, int y) {
			City city = new City(x, y);
			int i = this.random.Next(138);
			switch (i) {
				case 1: city.name = "Baltimore"; break;
				case 2: city.name = "Chicago"; break;
				case 3: city.name = "Cincinnati"; break;
				case 4: city.name = "New York City"; break;
				case 5: city.name = "Kansas City"; break;
				case 6: city.name = "San Diego"; break;
				case 7: city.name = "San Francisco"; break;
				case 8: city.name = "Paris"; break;
				case 9: city.name = "Berlin"; break;
				case 10: city.name = "Washington D.C."; break;
				case 11: city.name = "Cincinnati"; break;
				case 12: city.name = "Columbus"; break;
				case 13: city.name = "Philadelphia"; break;
				case 14: city.name = "New Orleans"; break;
				case 15: city.name = "Dallas"; break;
				case 16: city.name = "Salt Lake City"; break;
				case 17: city.name = "Houston"; break;
				case 18: city.name = "Los Angeles"; break;
				case 19: city.name = "Boston"; break;
				case 20: city.name = "Detroit"; break;
				case 21: city.name = "Toronto"; break;
				case 22: city.name = "Albany"; break;
				case 23: city.name = "Orlando"; break;
				case 24: city.name = "Anchorage"; break;
				case 25: city.name = "Brussels"; break;
				case 26: city.name = "London"; break;
				case 27: city.name = "Denver"; break;
				case 28: city.name = "San Antonio"; break;
				case 29: city.name = "Tampa"; break;
				case 30: city.name = "Miami"; break;
				case 31: city.name = "Atlanta"; break;
				case 32: city.name = "Providence"; break;
				case 33: city.name = "Tallahassee"; break;
				case 34: city.name = "Nashville"; break;
				case 35: city.name = "Las Vegas"; break;
				case 36: city.name = "Santa Fe"; break;
				case 37: city.name = "Boise"; break;
				case 38: city.name = "Seattle"; break;
				case 39: city.name = "Portland"; break;
				case 40: city.name = "Bismarck"; break;
				case 41: city.name = "Minneapolis"; break;
				case 42: city.name = "Milwaukee"; break;
				case 43: city.name = "Montreal"; break;
				case 44: city.name = "Winnipeg"; break;
				case 45: city.name = "Calgary"; break;
				case 46: city.name = "Birmingham"; break;
				case 47: city.name = "St. Louis"; break;
				case 48: city.name = "Little Rock"; break;
				case 49: city.name = "Oklahoma City"; break;
				case 50: city.name = "Indianapolis"; break;
				case 51: city.name = "Pittsburgh"; break;
				case 52: city.name = "Phoenix"; break;
				case 53: city.name = "Omaha"; break;
				case 54: city.name = "Wichita"; break;
				case 55: city.name = "Lexington"; break;
				case 56: city.name = "Louisville"; break;
				case 57: city.name = "Richmond"; break;
				case 58: city.name = "Charlotte"; break;
				case 59: city.name = "Savannah"; break;
				case 60: city.name = "Stalingrad"; break;
				case 61: city.name = "Leningrad"; break;
				case 62: city.name = "Moscow"; break;
				case 63: city.name = "Baghdad"; break;
				case 64: city.name = "Grand Rapids"; break;
				case 65: city.name = "Ottawa"; break;
				case 66: city.name = "Guatemala City"; break;
				case 67: city.name = "Mexico City"; break;
				case 68: city.name = "Havana"; break;
				case 69: city.name = "San Juan"; break;
				case 70: city.name = "Rio de Janeiro"; break;
				case 71: city.name = "Madrid"; break;
				case 72: city.name = "Barcelona"; break;
				case 73: city.name = "Marseilles"; break;
				case 74: city.name = "Amsterdam"; break;
				case 75: city.name = "Dublin"; break;
				case 76: city.name = "Glasgow"; break;
				case 77: city.name = "Frankfurt"; break;
				case 78: city.name = "Hamburg"; break;
				case 79: city.name = "Munich"; break;
				case 80: city.name = "Milan"; break;
				case 81: city.name = "Rome"; break;
				case 82: city.name = "Naples"; break;
				case 83: city.name = "Genoa"; break;
				case 84: city.name = "Belgrade"; break;
				case 85: city.name = "Zagreb"; break;
				case 86: city.name = "Sofia"; break;
				case 87: city.name = "Athens"; break;
				case 88: city.name = "Istanbul"; break;
				case 89: city.name = "Budapest"; break;
				case 90: city.name = "Warsaw"; break;
				case 91: city.name = "Kiev"; break;
				case 92: city.name = "Belarus"; break;
				case 93: city.name = "Stockholm"; break;
				case 94: city.name = "Oslo"; break;
				case 95: city.name = "Minsk"; break;
				case 96: city.name = "Kolkata"; break;
				case 97: city.name = "Beijing"; break;
				case 98: city.name = "Hong Kong"; break;
				case 99: city.name = "Tokyo"; break;
				case 100: city.name = "Mumbai"; break;
				case 101: city.name = "Bhopal"; break;
				case 102: city.name = "Delhi"; break;
				case 103: city.name = "Kabul"; break;
				case 104: city.name = "Mosul"; break;
				case 105: city.name = "Jerusalem"; break;
				case 106: city.name = "Damascus"; break;
				case 107: city.name = "Riyadh"; break;
				case 108: city.name = "Mecca"; break;
				case 109: city.name = "Dubai"; break;
				case 110: city.name = "Alexandria"; break;
				case 111: city.name = "Cairo"; break;
				case 112: city.name = "Tunis"; break;
				case 113: city.name = "Tripoli"; break;
				case 114: city.name = "Algiers"; break;
				case 115: city.name = "Beirut"; break;
				case 116: city.name = "Bangkok"; break;
				case 117: city.name = "Ho Chi Minh City"; break;
				case 118: city.name = "Hanoi"; break;
				case 119: city.name = "Chengdu"; break;
				case 120: city.name = "Guangzhou"; break;
				case 121: city.name = "Seoul"; break;
				case 122: city.name = "Shanghai"; break;
				case 123: city.name = "Phnom Penh"; break;
				case 124: city.name = "Mandalay"; break;
				case 125: city.name = "Kuala Lumpur"; break;
				case 126: city.name = "Singapore"; break;
				case 127: city.name = "Jakarta"; break;
				case 128: city.name = "Perth"; break;
				case 129: city.name = "Melbourne"; break;
				case 130: city.name = "Sydney"; break;
				case 131: city.name = "Brisbane"; break;
				case 132: city.name = "Hokkaido"; break;
				case 133: city.name = "Kyoto"; break;
				case 134: city.name = "Kagashima"; break;
				case 135: city.name = "Fukuoka"; break;
				case 136: city.name = "Murmansk"; break;
				case 137: city.name = "Turin"; break;
				case 138: city.name = "Lisbon"; break;
				default: city.name = "Cleveland"; break;
			}
			cities.Add(city);
			iNumCities = iEntitySize = cities.Count;
			SetDistances();
		}

		public void SetDistances() {
			int iMaxDistance = 0;
			iDistances = new int[iNumCities, iNumCities];
			for (int i = 0; i < iNumCities; i++) {
				for (int j = 0; j < iNumCities; j++) {
					iDistances[i, j] = (int) Math.Sqrt(Math.Pow(((City) cities[i]).x - ((City) cities[j]).x, 2) + Math.Pow(((City) cities[i]).y - ((City) cities[j]).y, 2));
					iMaxDistance = Math.Max(iMaxDistance, iDistances[i, j]);
				}
			}
			iMaxTrip = iMaxDistance * iNumCities;
		}

		public Route FindBestRoute() {
			int iBestDistance = 99999999;
			Route rBestRoute = null;
			foreach (Route r in this.lstEntities) {
				if (r.TotalDistance(iDistances) < iBestDistance) {
					iBestDistance = r.TotalDistance(iDistances);
					rBestRoute = r;
				}
			}
			return rBestRoute;
		}

		#endregion
		
	}

	public class Route : Entity {

		// This class adapts the Entity class as a Route.

		#region Initialization code

		public Route(int iNumCities) : base(iNumCities) {
			this.dFitness = 0;
		}

		#endregion

		#region Helper functions

		public int TotalDistance(int[,] iDistances) {
			int iTotalDistance = 0;
			if (contents.Length == 0)
				return 0;
			for (int i = 0; i < contents.Length - 1; i++)
				iTotalDistance += iDistances[(int) contents[i], (int) contents[i + 1]];
			iTotalDistance += iDistances[(int) contents[0], (int) contents[contents.Length - 1]];
			return iTotalDistance;
		}

		public double MeasureDistance(int[,] iDistances, int iMaxTrip) {
			return TotalDistance(iDistances);
		}

		#endregion
	}

	public class City {

		// This class represents a city - basically just a data structure.

		#region Variables and initializaiton code

		internal double x, y;
		internal string name;

		public City(int x, int y) {
			this.x = x;
			this.y = y;
		}

		#endregion

	}
}
